from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage


def paginate(queryset, current_page, per_page=6):
    # Paginate all items by per_page
    paginator = Paginator(queryset, per_page)
    # Try to get the ?page=x value
    try:
        # If the page exists and the ?page=x is an int
        items = paginator.page(current_page)
    except PageNotAnInteger:
        # If the ?page=x is not an int; show the first page
        items = paginator.page(1)
    except EmptyPage:
        # If the ?page=x is out of range (too high most likely)
        # Then return the last page
        items = paginator.page(paginator.num_pages)
    return items


def query_param_to_list(query_param, as_int=False):
    if query_param:
        # convert to list and remove empty items
        query_param_list = filter(None, query_param.split(','))

        if as_int:
            # convert to int
            query_param_list = map(int, query_param_list)

        try:
            return list(query_param_list)
        except ValueError:
            # TODO: Return error message
            pass

    return None


from django.urls import path,register_converter

from capeditor.views import AlertList,AlertDetail

class IdentifierConverter:
    regex = r'[A-Za-z0-9_-]+'

    def to_python(self, value):
        return value

    def to_url(self, value):
        return value

register_converter(IdentifierConverter, 'identifier')

urlpatterns = [
    path('caps.xml', AlertList.as_view()),
    path("<identifier:identifier>.xml", AlertDetail.as_view(), name="alert_by_id"),
]


from django import forms
from django.template.loader import render_to_string
import json
from django.utils.functional import cached_property


class BasemapPolygonWidget(forms.HiddenInput):
    template_name = "capeditor/widgets/basemap_polygon.html" 

    @cached_property
    def media(self):
        return forms.Media(
            css={
                "all": (
                    "https://cdnjs.cloudflare.com/ajax/libs/leaflet.draw/1.0.4/leaflet.draw.css",
                    "https://unpkg.com/leaflet@1.7.1/dist/leaflet.css",
                    
                )
            },
            js=(
                "https://unpkg.com/leaflet@1.7.1/dist/leaflet.js",
                "https://cdnjs.cloudflare.com/ajax/libs/leaflet.draw/1.0.4/leaflet.draw.js",
                "https://unpkg.com/togeojson@0.16.0",
                "https://unpkg.com/leaflet-filelayer@1.2.0",
                "https://kit.fontawesome.com/db8ac3c257.js"
            ),
        )

    def get_context(self, name, value, attrs=None):
        context = super().get_context(name, value, attrs)
        context['map_id'] = f'map-{name}'
        

        return context




from django.shortcuts import render

# Create your views here.
from rest_framework.response import Response

from rest_framework import generics,renderers
from capeditor.models import Alert
from capeditor.serializers import AlertSerializer
from capeditor.renderers import CustomXMLRenderer
from django.http import JsonResponse
from django.shortcuts import get_object_or_404
from rest_framework import status
from rest_framework.decorators import api_view
from django.views.decorators.csrf import csrf_exempt


class AlertList(generics.ListAPIView):
    queryset = Alert.objects.all()
    serializer_class = AlertSerializer
    renderer_classes = (CustomXMLRenderer,)

class AlertDetail(generics.RetrieveAPIView):
    queryset = Alert.objects.all()
    serializer_class = AlertSerializer
    renderer_classes = (CustomXMLRenderer,)
    lookup_field = 'identifier'

    # def get(self, request, format=None):
    #     pages = Alert.objects.all()
    #     serializer = AlertSerializer(pages, many=True)
    #     return Response(serializer.data)


# @api_view(["GET"])
# def get_alert_by_id(request, identifier):
#     alert = get_object_or_404(Alert, identifier=identifier)
#     alert_serializer = AlertSerializer(alert)

#     return JsonResponse(alert_serializer.data, safe=False)


from django.template.defaultfilters import stringfilter
from django import template

register = template.Library()

@stringfilter
def remove_param(query_string, key):
    """
    Removes a given parameter from a query_string.

    For example, to delete page from "page=1&page=2&year=2020"

        {{ "page=1&page=2&year=2020"|remove_param:"page" }}

        returns year=2020

    """

    query_string = query_string.replace("&amp;", "&")

    params = query_string.split("&")

    url = ""

    for param in params:
        if param:
            v = param.split('=')
            if v[0] != key:
                url += param + "&"
    return url


register.filter(remove_param)


import uuid
import hashlib

from django.utils.functional import cached_property
from django.contrib.gis.db import models
from django.core.validators import MaxValueValidator, MinValueValidator
from wagtail.models import Page
from wagtail.models import Orderable
from modelcluster.fields import ParentalKey
from wagtail.fields import RichTextField
from modelcluster.models import ClusterableModel
from datetime import datetime
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
# from Inlinepanel.edit_handlers import InlinePanel
from django.utils.text import slugify
from capeditor.utils import  paginate, query_param_to_list

from wagtail.admin.panels import MultiFieldPanel, FieldPanel, InlinePanel,FieldRowPanel
from capeditor.widgets import  BasemapPolygonWidget
        
            
# Create your models here.
class AlertList(Page):
    template = "capeditor/alert_index.html"
    ajax_template = 'capeditor/alert_list_include.html'

    subpage_types = [
        'capeditor.Alert',  # appname.ModelName
    ]
    parent_page_type = [
        'wagtailcore.Page'  # appname.ModelName
    ]
    max_count = 1

    alerts_per_page = models.PositiveIntegerField(default=6, validators=[
        MinValueValidator(6),
        MaxValueValidator(20),
    ], help_text=_("How many of this products should be visible on the landing page filter section ?"),
    verbose_name=_("Alerts Per Page")
    )

    content_panels = Page.content_panels +[
        FieldPanel('alerts_per_page'),
    ]

    @property
    def filters(self):

        URGENCIES = {
            'Immediate':_('Immediate'),
            'Expected':_('Expected'),
            'Future':_('Future'),
            'Past':_('Past'),
            'Unknown':_('Unknown'),
        }
        
        SEVERITIES = {
            'Extreme': _("Extreme - Extraordinary threat to life or property"),
            'Severe': _("Severe - Significant threat to life or property"),
            'Moderate': _("Moderate - Possible threat to life or property"),
            'Minor': _("Minor - Minimal to no known threat to life or property"),
            'Unknown': _("Unknown - Severity unknown"),
        }

        CERTAINTIES = {
            'Observed': _("Observed - Determined to have occurred or to be ongoing"),
            'Likely': _("Likely - Likely (percentage > ~50%)"),
            'Possible': _("Possible - Possible but not likely (percentage <= ~50%)"),
            'Unlikely': _("Unlikely - Not expected to occur (percentage ~ 0)"),
            'Unknown': _("Unknown - Certainty unknown"),
        }
        return {'urgency': URGENCIES, 'severity':SEVERITIES, 'certainty': CERTAINTIES}

    @property
    def all_alerts(self):
        return Alert.objects.live()

    def filter_alerts(self, request):
        alerts = self.all_alerts

        
        urgency = query_param_to_list(request.GET.get("urgency"), as_int=False)
        severity = query_param_to_list(request.GET.get("severity"), as_int=False)
        certainty = query_param_to_list(request.GET.get("certainty"), as_int=False)

        filters = models.Q()

        if urgency:
            filters &= models.Q(alert_info__urgency__in=urgency)
        if severity:
            filters &= models.Q(alert_info__severity__in=severity)
        if certainty:
            filters &= models.Q(alert_info__certainty__in=certainty)

        return alerts.filter(filters)

    def filter_and_paginate_alerts(self, request):
        page = request.GET.get('page')

        filtered_alerts = self.filter_alerts(request)

        paginated_alerts = paginate(filtered_alerts, page, self.alerts_per_page)

        return paginated_alerts


    def get_context(self, request, *args, **kwargs):
        context = super(AlertList,
                        self).get_context(request, *args, **kwargs)

        context['alerts'] = self.filter_and_paginate_alerts(request)
        context['latest_alert'] = self.all_alerts[0]

        return context
    

class Alert(Page):

    subpage_types = [
    ]
    parent_page_type = [
        'capeditor.AlertList'  # appname.ModelName
    ]

    template = "capeditor/alert_detail.html"

    STATUS_CHOICES = (
        ("Draft", _("Draft - A preliminary template or draft, not actionable in its current form")),
        ("Actual", _("Actual - Actionable by all targeted recipients")),
        ("Test", _("Test - Technical testing only, all recipients disregard")),
        ("Exercise",
         _("Exercise - Actionable only by designated exercise participants; exercise identifier SHOULD appear in note")),
        ("system", _("System - For messages that support alert network internal functions")),
    )

    MESSAGE_TYPE_CHOICES = (
        ('Alert', _("Alert - Initial information requiring attention by targeted recipients")),
        ('Update', _("Update - Updates and supercedes the earlier message(s) identified in referenced alerts")),
        ('Cancel', _("Cancel - Cancels the earlier message(s) identified in references")),
        ('Ack', _("Acknowledge - Acknowledges receipt and acceptance of the message(s) identified in references field")),
        ('Error',
         _("Error -  Indicates rejection of the message(s) identified in references; explanation SHOULD "
         "appear in note field")),
    )

    SCOPE_CHOICES = (
        ('Public', _("Public - For general dissemination to unrestricted audiences")),
        ('Restricted',
         _("Restricted - For dissemination only to users with a known operational requirement as in the restriction field")),
        ('Private', _("Private - For dissemination only to specified addresses as in the addresses field in the alert")),
    )

  
    identifier = models.UUIDField(default=uuid.uuid4, editable=False,
                                  help_text=_("Unique ID. Auto generated on creation."), verbose_name=_("Identifier"))
    sender = models.EmailField(max_length=255,
                              help_text=_(" Identifies the originator of an alert. "
                                        "This can be an email of the institution for example"), default='grace@gmail.com', verbose_name=_("Sender"))
    sent = models.DateTimeField(help_text=_("Time and date of origination of the alert"), default=timezone.now, verbose_name=_("Sent"))
    status = models.CharField(max_length=50, choices=STATUS_CHOICES,
                              help_text=_("The code denoting the appropriate handling of the alert"), default='Actual', verbose_name=_("Status"))
    msgType = models.CharField(max_length=100, choices=MESSAGE_TYPE_CHOICES,
                                    help_text=_("The code denoting the nature of the alert message"),  default='Alert', verbose_name=_("Message Type"))
    scope = models.CharField(max_length=100,
                             choices=SCOPE_CHOICES,
                             help_text=_("The code denoting the intended distribution of the alert message"),  default='Public', verbose_name=_("Scope"))
    source = models.TextField(blank=True, null=True, help_text=_("The text identifying the source of the alert message"), verbose_name=_("Source"))
    restriction = models.TextField(blank=True, null=True,
                                   help_text=_("The text describing the rule for limiting distribution of the "
                                             "restricted alert message. Used when scope value is Restricted"), verbose_name=_("Restriction"))
    code = models.CharField(max_length=100, blank=True, null=True,
                            help_text=_("The code denoting the special handling of the alert message"), verbose_name=_("Code"))
    note = models.TextField(blank=True, null=True,
                            help_text=_("The text describing the purpose or significance of the alert message."
                                      "The message note is primarily intended for use with "
                                      "<status> 'Exercise' and <msgType> 'Error'"), verbose_name=_("Note"))
                                                         
    content_panels = Page.content_panels + [

        MultiFieldPanel([
            FieldRowPanel([
            FieldPanel('sender'),
            FieldPanel('sent'),
            ]),
            FieldPanel('source'),
            
            FieldPanel('status',),
            FieldPanel('msgType', classname="message"),
            InlinePanel('references', heading=_("Earlier Reference Alerts -  If applicable"), label=_("Alert"), classname="references"),

            FieldPanel('note', classname='note'),

            FieldPanel('scope'),
            FieldPanel('restriction',classname="restriction" ),
            InlinePanel('addresses', heading=_("Intended Recipients (If scope is Private) "), label=_("Recipient"), classname="addresses"),


        ], heading=_("Alert Identification (Sender, Message Type, Scope)")),

        MultiFieldPanel([
            InlinePanel('alert_info', label=_("Alert Information"),  classname="collapsed")
        ], heading=_("Alert Info"))
    ]

    subpage_types = [
        # 'capeditor.Alert',  # appname.ModelName
    ]
    parent_page_type = [
        'wagtailcore.Page'  # appname.ModelName
    ]

class AlertAddress(Orderable):
    alert = ParentalKey('Alert', related_name="addresses", verbose_name=_("Alert"))
    name = models.TextField(help_text=_("Name of the recipient"))
    address = models.EmailField(blank=True, null=True, help_text=_("Email"), verbose_name=_("Email address"))

    def __str__(self):
        return self.name


class AlertReference(Orderable):
    alert = ParentalKey('Alert', related_name='references', verbose_name=_("Alert"))
    ref_alert = models.ForeignKey('Alert', blank=True, null=True, on_delete=models.PROTECT,
                                  help_text=_("Earlier alert referenced by this alert"), verbose_name=_("Reference Alert"))

    def __str__(self):
        return f'{self.ref_alert.sender},{self.ref_alert.identifier},{self.ref_alert.sent}'
    
    @property
    def reference(self):
        return f'{self.ref_alert.sender},{self.ref_alert.identifier},{self.ref_alert.sent}'



class AlertIncident(Orderable):
    alert = ParentalKey('Alert', related_name='incidents', on_delete=models.CASCADE, verbose_name=_("Alert"))
    title = models.CharField(max_length=255, help_text=_("Title of the incident referent of the alert"), verbose_name=_("Title"))
    description = models.TextField(help_text=_("Description of the incident"), verbose_name=_("Description"))

    def __str__(self):
        return self.title

class AlertInfo(ClusterableModel):

    LANGUAGE_CHOICES = (
        ('en', _("English")),
    )

    CATEGORY_CHOICES = (
        ('Geo', _("Geophysical")),
        ('Met', _("Meteorological")),
        ('Safety', _("General emergency and public safety")),
        ('Security', _("Law enforcement, military, homeland and local/private security")),
        ('Rescue', _("Rescue and recovery")),
        ('Fire', _("Fire suppression and rescue")),
        ('Health', _("Medical and public health")),
        ('Env', _("Pollution and other environmental")),
        ('Transport', _("Public and private transportation")),
        ('Infra', _("Utility, telecommunication, other non-transport infrastructure")),
        ('Cbrne', _("Chemical, Biological, Radiological, Nuclear or High-Yield Explosive threat or attack")),
        ('Other', _("Other events")),
    )

    URGENCY_CHOICES = (
        ('Immediate', _("Immediate - Responsive action SHOULD be taken immediately")),
        ('Expected', _("Expected - Responsive action SHOULD be taken soon (within next hour)")),
        ('Future', _("Future - Responsive action SHOULD be taken in the near future")),
        ('Past', _("Past - Responsive action is no longer required")),
        ('Unknown', _("Unknown - Urgency not known")),
    )

    SEVERITY_CHOICES = (
        ('Extreme', _("Extreme - Extraordinary threat to life or property")),
        ('Severe', _("Severe - Significant threat to life or property")),
        ('Moderate', _("Moderate - Possible threat to life or property")),
        ('Minor', _("Minor - Minimal to no known threat to life or property")),
        ('Unknown', _("Unknown - Severity unknown")),
    )

    CERTAINTY_CHOICES = (
        ('Observed', _("Observed - Determined to have occurred or to be ongoing")),
        ('Likely', _("Likely - Likely (percentage > ~50%)")),
        ('Possible', _("Possible - Possible but not likely (percentage <= ~50%)")),
        ('Unlikely', _("Unlikely - Not expected to occur (percentage ~ 0)")),
        ('Unknown', _("Unknown - Certainty unknown")),
    )

    alert = ParentalKey('Alert', related_name="alert_info", )

    language = models.CharField(max_length=100, choices=LANGUAGE_CHOICES, default='en', blank=True, null=True,
                                help_text=_("The code denoting the language of the alert message"), verbose_name=_("Language"))
    category = models.CharField(max_length=100,  default='Met',
                                choices=CATEGORY_CHOICES,
                                help_text=_("The code denoting the category of the subject event of the alert message"), verbose_name=_("Category"))
    event = models.CharField(max_length=100,
                             help_text=_("The text denoting the type of the subject event of the alert message"), blank=True, null=True, verbose_name=_("Event"))
    urgency = models.CharField(max_length=100,
                               choices=URGENCY_CHOICES,
                               verbose_name=_("Urgency"),
                               help_text=_("The code denoting the urgency of the subject event of the alert message"), default="Immediate")
    severity = models.CharField(max_length=100,
                                choices=SEVERITY_CHOICES,
                                verbose_name=_("Severity"),
                                help_text=_("The code denoting the severity of the subject event of the alert message"), default="Extreme")
    certainty = models.CharField(max_length=100,
                                 choices=CERTAINTY_CHOICES,
                                 verbose_name=_("Certainty"),
                                 help_text=_("The code denoting the certainty of the subject event of the alert message"), default="Likely")
    audience = models.TextField(blank=True, null=True,
                                verbose_name=_("Audience"),
                                help_text=_("The text describing the intended audience of the alert message"))

    effective = models.DateTimeField(blank=True, null=True,
                                     verbose_name=_("Effective"),
                                     help_text=_("The effective time of the information of the alert message"))
    onset = models.DateTimeField(blank=True, null=True,
                                 verbose_name=_("Onset"),
                                 help_text=_("The expected time of the beginning of the subject event "
                                           "of the alert message"))
    expires = models.DateTimeField(blank=True, null=True,
                                   verbose_name=_("Expires"),
                                   help_text=_("The expiry time of the information of the alert message"))
    headline = models.TextField(blank=True, null=True,verbose_name=_("Headline"), help_text=_("The text headline of the alert message"))
    description = models.TextField(blank=True, null=True,verbose_name=_("Description"),
                                   help_text=_("The text describing the subject event of the alert message"))
    instruction = models.TextField(blank=True, null=True,verbose_name=_("Instruction"),
                                   help_text=_("The text describing the recommended action to be taken by "
                                             "recipients of the alert message"))
    web = models.URLField(blank=True, null=True,verbose_name=_("Web"),
                          help_text=_("The identifier of the hyperlink associating "
                                    "additional information with the alert message"))
    contact = models.TextField(blank=True, null=True,verbose_name=_("Contact"),
                               help_text=_("The text describing the contact for follow-up and "
                                         "confirmation of the alert message"))

    
    panels = [

        MultiFieldPanel([
            FieldRowPanel([
            FieldPanel('language'),
            FieldPanel('event'),

            ]),
            FieldPanel('category'),
            FieldPanel('urgency'),
            FieldPanel('severity'),
            FieldPanel('certainty'),
            InlinePanel('response_types', heading="Response Types ", label="Response Type"),        
            FieldPanel('audience'),
            # FieldPanel('event_codes'),
            FieldRowPanel([
                FieldPanel('effective'),
                FieldPanel('onset'),
            ]),
            FieldPanel('expires'),

            
        ], heading=_("Alert Categorization (Category, Urgency, Severity, Certainity, Response & Dates)"), classname="collapsed"),

        MultiFieldPanel([
            FieldPanel('headline'),
            FieldPanel('description'),
            FieldPanel('instruction'),

            FieldRowPanel([
                FieldPanel('web'),
                FieldPanel('contact'),
            ])
        ], heading =_("Alert Delivery Message (Headline, Description, Intsructions, Contact & Website)"), classname="collapsed"),


        MultiFieldPanel([
        InlinePanel('resources', heading=_("Alert Resources "), label=_("Resource")),
        ],  classname="collapsed"),

        MultiFieldPanel([
            InlinePanel('alert_areas',  heading=_("Alert Areas "),label=_("Alert Area")),
        ], classname="collapsed"),

        
    ]

    @property
    def is_expired(self):
        difference = (timezone.now() - self.expires).days
        if difference >= 0:
            return True
        return False


class AlertResponseType(Orderable):
    RESPONSE_TYPE_CHOICES = (
        ("Shelter", _("Shelter - Take shelter in place or per instruction")),
        ("Evacuate", _("Evacuate - Relocate as instructed in the instruction")),
        ("Prepare", _("Prepare - Relocate as instructed in the instruction")),
        ("Execute", _("Execute - Execute a pre-planned activity identified in instruction")),
        ("Avoid", _("Avoid - Avoid the subject event as per the instruction")),
        ("Monitor", _("Monitor - Attend to information sources as described in instruction")),
        ("Assess", _("Assess - Evaluate the information in this message - DONT USE FOR PUBLIC ALERTS")),
        ("AllClear",
         _("All Clear - The subject event no longer poses a threat or concern and any follow on action is described in instruction")),
        ("None", _("No action recommended")),
    )

    alert = ParentalKey('AlertInfo', related_name='response_types', null=True)
    response_type = models.CharField(max_length=100, choices=RESPONSE_TYPE_CHOICES,verbose_name=_("Response type"),
                                     help_text=_("The code denoting the type of action recommended for the "
                                               "target audience"))

    def __str__(self) -> str:
        return self.response_type


class AlertResource(Orderable):
    alert_info = ParentalKey('AlertInfo', related_name='resources', null=True)
    mimeType = models.CharField(max_length=100, blank=True, null=True,
                                     help_text=_("Resource type whether is image, file etc"))
    resourceDesc = models.TextField(help_text=_("The text describing the type and content of the resource file"))
    file = models.ForeignKey(
        'wagtaildocs.Document',
        help_text=_("File, Document etc"),
        verbose_name=_("File"),
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
    )
    uri = models.URLField(blank=True, null=True, help_text=_("The identifier of the hyperlink for the resource file"), verbose_name=_('Link'))
    derefUri = models.TextField(blank=True, null=True,
                                help_text=_("The base-64 encoded data content of the resource file"))
    digest = models.TextField(blank=True, null=True,
                              help_text=_("The code representing the digital digest ('hash') computed "
                                        "from the resource file"))
    
    size = models.IntegerField(null=True, blank=True)

    panels = [
        FieldPanel('resourceDesc'),
        FieldPanel('file'),
    ]

    @property
    def mime_type(self):
        return None

    
    def save(self ,*args, **kwargs):
        self.size = self.file.file.size
        self.mimeType = self.file.content_type
        self.uri = self.file.url
        with open(self.file.file.path, 'rb') as file:
            document_content = file.read()
        self.digest = hashlib.sha256(document_content).hexdigest()

        return super(AlertResource, self).save(*args, **kwargs)


class AlertArea(ClusterableModel):
    alert_info = ParentalKey('AlertInfo', related_name='alert_areas', null=True)
    areaDesc = models.CharField(max_length=100, help_text=_("The text describing the affected area of the alert message"),
                                 verbose_name=_("Affected areas / Regions"),null=True)
   
    area = models.PolygonField(help_text=_("The paired values of points defining a polygon that delineates the affected "
                                         "area of the alert message"), verbose_name=_("Area"), null=True, srid=4326)

    altitude = models.CharField(max_length=100,
                                blank=True,
                                null=True,
                                help_text=_("The specific or minimum altitude of the affected area of the alert message"),
                                verbose_name=_("Altitude"))
    ceiling = models.CharField(max_length=100,
                               blank=True,
                               null=True,
                               verbose_name=_("Ceiling"),
                               help_text=_("The maximum altitude of the affected area of the alert message."
                                         "MUST NOT be used except in combination with the altitude element. "))
                          
    panels = [
        FieldPanel('areaDesc'),
        FieldPanel('area', widget=BasemapPolygonWidget() ),
        InlinePanel('geocodes', label="Geocode", heading="Geocodes "),
        FieldPanel('altitude'),
        FieldPanel('ceiling'),
    ]   

class AlertEventCode(Orderable):
    alert_info = ParentalKey('AlertInfo', related_name='event_codes', null=True)
    name = models.CharField(max_length=100, help_text=_("Name for the event code"), verbose_name=_("Name"))
    value = models.CharField(max_length=255, help_text=_("Value of the event code"), verbose_name=_("Value"))


class AlertGeocode(Orderable):
    alert_info = ParentalKey('AlertArea', related_name='geocodes', null=True)
    name = models.CharField(max_length=100, help_text=_("Name for the geocode"), verbose_name=_("Name"))
    value = models.CharField(max_length=255, help_text=_("Value of the geocode"), verbose_name=_("Value"))



# from io import StringIO
# from rest_framework_xml.renderers import XMLRenderer
import six
import datetime

from rest_framework_xml.renderers import XMLRenderer
from xml.etree.ElementTree import Element, tostring

class CustomXMLRenderer(XMLRenderer):

    format = 'xml'
    root_tag_name = 'alert'

    def _recursive_serialize_dict(self, value, xml):
        for key, value in value.items():
            if isinstance(value, list):
                for item in value:
                    element = Element(key)
                    self._recursive_serialize(item, element)
                    xml.append(element)
            else:
                element = Element(key)
                self._recursive_serialize(value, element)
                xml.append(element)
    
    def _recursive_serialize(self, value, xml):
        
        if isinstance(value, (list, tuple)):
            self._recursive_serialize_list(value, xml)
        elif isinstance(value, dict):
            self._recursive_serialize_dict(value, xml)
        elif isinstance(value, datetime.datetime):
            xml.text = value.isoformat()
        elif isinstance(value, (six.integer_types, float)):
            xml.text = six.text_type(value)
        elif isinstance(value, six.text_type):
            xml.text = value
        elif value is None:
            xml.set('nil', 'true')
        else:
            xml.text = six.text_type(value)

    def render(self, data, accepted_media_type=None, renderer_context=None):
        """
        Render `data` into XML.
        """
        if data is None:
            return ''
        elif isinstance(data, list):
            
            
            root = Element('feed')
            root.set('xmlns', 'urn:oasis:names:tc:emergency:cap:1.2')
            
            for item in data:
                element = Element('alert')
                self._recursive_serialize(item, element)
                
                root.append(element)

            # if len(root) == 0:

            #     return ''.join([tostring(child, encoding='unicode') for child in root])

            return tostring(root, encoding='unicode')
        elif isinstance(data, dict):
            root = Element('alert')
            root.set('xmlns', 'urn:oasis:names:tc:emergency:cap:1.2')

            self._recursive_serialize_dict(data, root)
            # if len(root) > 1:

            #     return ''.join([tostring(child, encoding='unicode') for child in root])

            return tostring(root, encoding='unicode')
        else:
            return data


        # def render(self, data, accepted_media_type=None, renderer_context=None):

        # if data is None:
        #     return ''
            
        # elif isinstance(data, list):
        
        #     root = Element('feed')
        #     for item in data:
        #         element = Element('alert')
        #         self._recursive_serialize(item, element)
                
        #         root.append(element)
        #     # remove the root tag and return its child nodes as XML
        #     return ''.join([tostring(child, encoding='unicode') for child in root])
        # elif isinstance(data, dict):
        #     # if 'link' in data:
        #     #     link = data['link']
        #     #     del link

        #     root = Element('feed')
        #     self._recursive_serialize_dict(data, root)
        #     # remove the root tag and return its child nodes as XML
        #     return ''.join([tostring(child, encoding='unicode') for child in root])
        # else:
        #     return data
        




from wagtail import hooks
from django.utils.html import format_html
from django.templatetags.static import static
from capeditor.models import Alert
from wagtail.contrib.modeladmin.options import (
    ModelAdmin, 
    modeladmin_register
)

@hooks.register("insert_editor_js")
def insert_editor_js():
    return format_html(
        '<script src="{}"></script>',static("js/hide_attributes.js"),
    )

@hooks.register("insert_global_admin_css")
def insert_global_admin_css():
    return format_html(
        '<link rel="stylesheet" type="text/css" href="{}">',
        static("css/admin.css"),
    )

class CAPAdmin(ModelAdmin):
    model = Alert
    menu_label = 'CAP Alerts'
    menu_icon = 'cog'
    menu_order = 200 
    add_to_settings_menu = False
    exclude_from_explorer = False

modeladmin_register(CAPAdmin)


# Generated by Django 4.1.7 on 2023-05-02 14:56

import django.contrib.gis.db.models.fields
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import modelcluster.fields
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtaildocs', '0012_uploadeddocument'),
        ('wagtailcore', '0078_referenceindex'),
    ]

    operations = [
        migrations.CreateModel(
            name='Alert',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.page')),
                ('identifier', models.UUIDField(default=uuid.uuid4, editable=False, help_text='Unique ID. Auto generated on creation.', verbose_name='Identifier')),
                ('sender', models.EmailField(default='grace@gmail.com', help_text=' Identifies the originator of an alert. This can be an email of the institution for example', max_length=255, verbose_name='Sender')),
                ('sent', models.DateTimeField(default=django.utils.timezone.now, help_text='Time and date of origination of the alert', verbose_name='Sent')),
                ('status', models.CharField(choices=[('Draft', 'Draft - A preliminary template or draft, not actionable in its current form'), ('Actual', 'Actual - Actionable by all targeted recipients'), ('Test', 'Test - Technical testing only, all recipients disregard'), ('Exercise', 'Exercise - Actionable only by designated exercise participants; exercise identifier SHOULD appear in note'), ('system', 'System - For messages that support alert network internal functions')], default='Actual', help_text='The code denoting the appropriate handling of the alert', max_length=50, verbose_name='Status')),
                ('msgType', models.CharField(choices=[('Alert', 'Alert - Initial information requiring attention by targeted recipients'), ('Update', 'Update - Updates and supercedes the earlier message(s) identified in referenced alerts'), ('Cancel', 'Cancel - Cancels the earlier message(s) identified in references'), ('Ack', 'Acknowledge - Acknowledges receipt and acceptance of the message(s) identified in references field'), ('Error', 'Error -  Indicates rejection of the message(s) identified in references; explanation SHOULD appear in note field')], default='Alert', help_text='The code denoting the nature of the alert message', max_length=100, verbose_name='Message Type')),
                ('scope', models.CharField(choices=[('Public', 'Public - For general dissemination to unrestricted audiences'), ('Restricted', 'Restricted - For dissemination only to users with a known operational requirement as in the restriction field'), ('Private', 'Private - For dissemination only to specified addresses as in the addresses field in the alert')], default='Public', help_text='The code denoting the intended distribution of the alert message', max_length=100, verbose_name='Scope')),
                ('source', models.TextField(blank=True, help_text='The text identifying the source of the alert message', null=True, verbose_name='Source')),
                ('restriction', models.TextField(blank=True, help_text='The text describing the rule for limiting distribution of the restricted alert message. Used when scope value is Restricted', null=True, verbose_name='Restriction')),
                ('code', models.CharField(blank=True, help_text='The code denoting the special handling of the alert message', max_length=100, null=True, verbose_name='Code')),
                ('note', models.TextField(blank=True, help_text="The text describing the purpose or significance of the alert message.The message note is primarily intended for use with <status> 'Exercise' and <msgType> 'Error'", null=True, verbose_name='Note')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='AlertArea',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('areaDesc', models.CharField(help_text='The text describing the affected area of the alert message', max_length=100, null=True, verbose_name='Affected areas / Regions')),
                ('area', django.contrib.gis.db.models.fields.PolygonField(help_text='The paired values of points defining a polygon that delineates the affected area of the alert message', null=True, srid=4326, verbose_name='Area')),
                ('altitude', models.CharField(blank=True, help_text='The specific or minimum altitude of the affected area of the alert message', max_length=100, null=True, verbose_name='Altitude')),
                ('ceiling', models.CharField(blank=True, help_text='The maximum altitude of the affected area of the alert message.MUST NOT be used except in combination with the altitude element. ', max_length=100, null=True, verbose_name='Ceiling')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AlertInfo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language', models.CharField(blank=True, choices=[('en', 'English')], default='en', help_text='The code denoting the language of the alert message', max_length=100, null=True, verbose_name='Language')),
                ('category', models.CharField(choices=[('Geo', 'Geophysical'), ('Met', 'Meteorological'), ('Safety', 'General emergency and public safety'), ('Security', 'Law enforcement, military, homeland and local/private security'), ('Rescue', 'Rescue and recovery'), ('Fire', 'Fire suppression and rescue'), ('Health', 'Medical and public health'), ('Env', 'Pollution and other environmental'), ('Transport', 'Public and private transportation'), ('Infra', 'Utility, telecommunication, other non-transport infrastructure'), ('Cbrne', 'Chemical, Biological, Radiological, Nuclear or High-Yield Explosive threat or attack'), ('Other', 'Other events')], default='Met', help_text='The code denoting the category of the subject event of the alert message', max_length=100, verbose_name='Category')),
                ('event', models.CharField(blank=True, help_text='The text denoting the type of the subject event of the alert message', max_length=100, null=True, verbose_name='Event')),
                ('urgency', models.CharField(choices=[('Immediate', 'Immediate - Responsive action SHOULD be taken immediately'), ('Expected', 'Expected - Responsive action SHOULD be taken soon (within next hour)'), ('Future', 'Future - Responsive action SHOULD be taken in the near future'), ('Past', 'Past - Responsive action is no longer required'), ('Unknown', 'Unknown - Urgency not known')], default='Immediate', help_text='The code denoting the urgency of the subject event of the alert message', max_length=100, verbose_name='Urgency')),
                ('severity', models.CharField(choices=[('Extreme', 'Extreme - Extraordinary threat to life or property'), ('Severe', 'Severe - Significant threat to life or property'), ('Moderate', 'Moderate - Possible threat to life or property'), ('Minor', 'Minor - Minimal to no known threat to life or property'), ('Unknown', 'Unknown - Severity unknown')], default='Extreme', help_text='The code denoting the severity of the subject event of the alert message', max_length=100, verbose_name='Severity')),
                ('certainty', models.CharField(choices=[('Observed', 'Observed - Determined to have occurred or to be ongoing'), ('Likely', 'Likely - Likely (percentage > ~50%)'), ('Possible', 'Possible - Possible but not likely (percentage <= ~50%)'), ('Unlikely', 'Unlikely - Not expected to occur (percentage ~ 0)'), ('Unknown', 'Unknown - Certainty unknown')], default='Likely', help_text='The code denoting the certainty of the subject event of the alert message', max_length=100, verbose_name='Certainty')),
                ('audience', models.TextField(blank=True, help_text='The text describing the intended audience of the alert message', null=True, verbose_name='Audience')),
                ('effective', models.DateTimeField(blank=True, help_text='The effective time of the information of the alert message', null=True, verbose_name='Effective')),
                ('onset', models.DateTimeField(blank=True, help_text='The expected time of the beginning of the subject event of the alert message', null=True, verbose_name='Onset')),
                ('expires', models.DateTimeField(blank=True, help_text='The expiry time of the information of the alert message', null=True, verbose_name='Expires')),
                ('headline', models.TextField(blank=True, help_text='The text headline of the alert message', null=True, verbose_name='Headline')),
                ('description', models.TextField(blank=True, help_text='The text describing the subject event of the alert message', null=True, verbose_name='Description')),
                ('instruction', models.TextField(blank=True, help_text='The text describing the recommended action to be taken by recipients of the alert message', null=True, verbose_name='Instruction')),
                ('web', models.URLField(blank=True, help_text='The identifier of the hyperlink associating additional information with the alert message', null=True, verbose_name='Web')),
                ('contact', models.TextField(blank=True, help_text='The text describing the contact for follow-up and confirmation of the alert message', null=True, verbose_name='Contact')),
                ('alert', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='alert_info', to='capeditor.alert')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AlertList',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.page')),
                ('alerts_per_page', models.PositiveIntegerField(default=6, help_text='How many of this products should be visible on the landing page filter section ?', validators=[django.core.validators.MinValueValidator(6), django.core.validators.MaxValueValidator(20)], verbose_name='Alerts Per Page')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='AlertResponseType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('response_type', models.CharField(choices=[('Shelter', 'Shelter - Take shelter in place or per instruction'), ('Evacuate', 'Evacuate - Relocate as instructed in the instruction'), ('Prepare', 'Prepare - Relocate as instructed in the instruction'), ('Execute', 'Execute - Execute a pre-planned activity identified in instruction'), ('Avoid', 'Avoid - Avoid the subject event as per the instruction'), ('Monitor', 'Monitor - Attend to information sources as described in instruction'), ('Assess', 'Assess - Evaluate the information in this message - DONT USE FOR PUBLIC ALERTS'), ('AllClear', 'All Clear - The subject event no longer poses a threat or concern and any follow on action is described in instruction'), ('None', 'No action recommended')], help_text='The code denoting the type of action recommended for the target audience', max_length=100, verbose_name='Response type')),
                ('alert', modelcluster.fields.ParentalKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='response_types', to='capeditor.alertinfo')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AlertResource',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('mimeType', models.CharField(blank=True, help_text='Resource type whether is image, file etc', max_length=100, null=True)),
                ('resourceDesc', models.TextField(help_text='The text describing the type and content of the resource file')),
                ('uri', models.URLField(blank=True, help_text='The identifier of the hyperlink for the resource file', null=True, verbose_name='Link')),
                ('derefUri', models.TextField(blank=True, help_text='The base-64 encoded data content of the resource file', null=True)),
                ('digest', models.TextField(blank=True, help_text="The code representing the digital digest ('hash') computed from the resource file", null=True)),
                ('size', models.IntegerField(blank=True, null=True)),
                ('alert_info', modelcluster.fields.ParentalKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='resources', to='capeditor.alertinfo')),
                ('file', models.ForeignKey(blank=True, help_text='File, Document etc', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtaildocs.document', verbose_name='File')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AlertReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('alert', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='references', to='capeditor.alert', verbose_name='Alert')),
                ('ref_alert', models.ForeignKey(blank=True, help_text='Earlier alert referenced by this alert', null=True, on_delete=django.db.models.deletion.PROTECT, to='capeditor.alert', verbose_name='Reference Alert')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AlertIncident',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('title', models.CharField(help_text='Title of the incident referent of the alert', max_length=255, verbose_name='Title')),
                ('description', models.TextField(help_text='Description of the incident', verbose_name='Description')),
                ('alert', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='incidents', to='capeditor.alert', verbose_name='Alert')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AlertGeocode',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('name', models.CharField(help_text='Name for the geocode', max_length=100, verbose_name='Name')),
                ('value', models.CharField(help_text='Value of the geocode', max_length=255, verbose_name='Value')),
                ('alert_info', modelcluster.fields.ParentalKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='geocodes', to='capeditor.alertarea')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AlertEventCode',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('name', models.CharField(help_text='Name for the event code', max_length=100, verbose_name='Name')),
                ('value', models.CharField(help_text='Value of the event code', max_length=255, verbose_name='Value')),
                ('alert_info', modelcluster.fields.ParentalKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='event_codes', to='capeditor.alertinfo')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='alertarea',
            name='alert_info',
            field=modelcluster.fields.ParentalKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='alert_areas', to='capeditor.alertinfo'),
        ),
        migrations.CreateModel(
            name='AlertAddress',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('name', models.TextField(help_text='Name of the recipient')),
                ('address', models.EmailField(blank=True, help_text='Email', max_length=254, null=True, verbose_name='Email address')),
                ('alert', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='addresses', to='capeditor.alert', verbose_name='Alert')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
    ]



from setuptools import setup, find_packages

setup(
    name='capeditor',
    version='0.1.3',
    description='Wagtail CAP Editor can be used as standalone or integrated into website',
    author='Grace Amondi',
    author_email='miswa.grace@gmail.com',
    install_requires=[
        'Django>=4.0.0',
        'djangorestframework-xml>=2.0.0',
        'six>=1.16.0',
        'wagtail>=4.0.0',
        'wagtail-cache>=2.2.0',
        'psycopg2-binary>=2.9.5',
        'wagtail-lazyimages>=0.1.5'
    ],
    include_package_data=True,
    packages=find_packages(),
)



from rest_framework import serializers
from capeditor.models import Alert, AlertArea, AlertInfo,AlertGeocode,AlertResponseType, AlertAddress, AlertReference, AlertResource
from django.urls import reverse
import datetime
import pytz

def parseTZ(date_str):
    dt = datetime.datetime.strptime(date_str, "%Y-%m-%dT%H:%M:%SZ")
    dt = dt.replace(tzinfo=pytz.UTC)
    date_str = dt.strftime("%Y-%m-%dT%H:%M:%S%z")
    date_str = date_str[:-2] + ':' + date_str[-2:]

    return date_str


class LatLonField(serializers.Field):
    def to_representation(self, value):
        # Check if the value is None or an empty string
        if not value:
            return None

        # Extract the latitude and longitude values from the polygon string
        polygon_str = str(value)
        polygon_str = polygon_str.replace('((', '(').replace('))', ')')

        start_index = polygon_str.find("(") + 1
        end_index = polygon_str.find(")")
        coords_str = polygon_str[start_index:end_index]
        lat_lon_pairs = coords_str.split(", ")

        lat_lon_ls = []
        for pair in lat_lon_pairs:
            lat_lon_ls.append(f'{pair.split(" ")[1]},{pair.split(" ")[0]}')

        # Return the latitude and longitude values as a dictionary
        return " ".join(lat_lon_ls)

        
class AlertAddressSerializer(serializers.ModelSerializer):
    # address_ls = AddressField(source="address")

    class Meta:
        model = AlertAddress
        fields = ['address']

class AlertResponseTypeSerializer(serializers.ModelSerializer):
    class Meta:
        model = AlertResponseType
        fields = ['response_type']

    def to_representation(self, instance):
        representation = super().to_representation(instance)
        # Modify the XML tag name
        representation[f'responseType'] = representation.pop('response_type')
        # request = self.context.get('request')
        # if request:
        #     representation['link'] = self.get_link(instance)
        # Modify the XML output as needed
        # representation['other_field'] = 'some value'
        return {k: v for k, v in representation.items() if v or v == 0 or v == False}


class AlertGeocodeSerializer(serializers.ModelSerializer):

    class Meta:
        model = AlertGeocode
        fields = ['name', 'value']

    def to_representation(self, instance):
        representation = super().to_representation(instance)
        # Modify the XML tag name
        representation[f'valueName'] = representation.pop('name')
        # request = self.context.get('request')
        # if request:
        #     representation['link'] = self.get_link(instance)
        # Modify the XML output as needed
        # representation['other_field'] = 'some value'
        return {k: v for k, v in representation.items() if v or v == 0 or v == False}


class AlertResourceSerializer(serializers.ModelSerializer):

    class Meta:
        model = AlertResource
        fields = ['resourceDesc', 'uri', 'digest', 'size', 'mimeType']

class AlertAreaSerializer(serializers.ModelSerializer):
    geocode = serializers.SerializerMethodField()
    polygon = LatLonField(source="area")

    class Meta:
        model = AlertArea
        fields = ['areaDesc', 'polygon', 'geocode', 'altitude', 'ceiling']
        # fields = '__all__'

    @staticmethod
    def get_geocode(obj):
        serializer = AlertGeocodeSerializer(obj.geocodes,  many=True)
        return serializer.data

    def to_representation(self, instance):
        representation = super().to_representation(instance)
      
        return {k: v for k, v in representation.items() if v or v == 0 or v == False}

class AlertInfoSerializer(serializers.ModelSerializer):
    area = serializers.SerializerMethodField()
    responseType = serializers.SerializerMethodField()
    description = serializers.CharField()
    resource = serializers.SerializerMethodField()

    class Meta:
        model = AlertInfo
        fields = ['language', 'category', 'event','responseType', 'urgency','severity', 'certainty', 'audience', 'effective', 'onset', 'expires', 'headline', 'description', 'instruction', 'web', 'contact', 'area','resource']
        # fields = '__all__'


    @staticmethod
    def get_resource(obj):
        serializer = AlertResourceSerializer(obj.resources,  many=True)
        return serializer.data

    @staticmethod
    def get_responseType(obj):
        serializer = AlertResponseTypeSerializer(obj.response_types, many=True)
        resp_ls = []
        for resp in serializer.data:
            resp_ls.append(resp['responseType'])

        return resp_ls

    @staticmethod
    def get_area(obj):
        serializer = AlertAreaSerializer(obj.alert_areas,  many=True)
        return serializer.data

    def to_representation(self, instance):
        representation = super().to_representation(instance)
        representation['description'] = str(representation['description'])
        representation['effective'] = parseTZ(representation['effective'])
        representation['onset'] = parseTZ(representation['onset'])
        representation['expires'] = parseTZ(representation['expires'])

                
        return {k: v for k, v in representation.items() if v or v == 0 or v == False}

class AlertReferenceSerializer(serializers.ModelSerializer):
    alerts = serializers.SerializerMethodField()
    class Meta:
        model = AlertReference
        fields = ['alerts']

    @staticmethod
    def get_alerts(obj):
        serializer = AlertSerializer(obj.ref_alert)

        data = serializer.data        

        return f"{data['sender']},{data['identifier']},{data['sent']}"

class AlertSerializer(serializers.ModelSerializer):

    info = serializers.SerializerMethodField()
    addresses = serializers.SerializerMethodField()
    references = serializers.SerializerMethodField()
    class Meta:
        model = Alert
        fields = ['identifier', 'sender', 'sent', 'status', 'msgType', 'scope', 'source', 'restriction', 'code', 'note', 'addresses', 'references','info']
        # fields = '__all__'
        # depth = 1
        
    @staticmethod
    def get_info(obj):
        serializer = AlertInfoSerializer(obj.alert_info,  many=True)
        return serializer.data

    @staticmethod
    def get_addresses(obj):
        serializer = AlertAddressSerializer(obj.addresses, many = True)
        address_ls = []

        if serializer.data:
            for data in serializer.data:
                if data['address']:
                    address_ls.append(data['address'])

            return ' '.join(address_ls)
        
        return None

    @staticmethod
    def get_references(obj):
        serializer = AlertReferenceSerializer(obj.references, many = True)

        reference_ls =[]
        if serializer.data:
            for data in serializer.data:
                if data['alerts']:
                    reference_ls.append(data['alerts'])

            return ' '.join(reference_ls)

        return serializer.data


    def to_representation(self, instance):
        representation = super().to_representation(instance)

        representation['sent'] = parseTZ(representation['sent'])
        # Modify the XML tag name
        # representation[f'msgType'] = representation.pop('message_type')

        # check for scope condition
        if representation[f'scope'] == 'Restricted':
            representation[f'addresses'] = None
        elif representation[f'scope'] == 'Private':
            representation[f'restriction'] = None
        elif representation[f'scope'] == 'Public':
            representation[f'restriction'] = None
            representation[f'addresses'] = None

        # check message type 
        if representation[f'msgType'] == 'Update' or representation[f'msgType'] == 'Cancel'  or representation[f'msgType'] == 'Ack':
            representation[f'note'] = None
        elif representation[f'msgType'] == 'Alert':
            representation[f'note'] = None
            representation[f'references'] = None

        # request = self.context.get('request')
        # if request:
        #     representation['link'] = self.get_link(instance)
        # Modify the XML output as needed
        # representation['other_field'] = 'some value'
        return {k: v for k, v in representation.items() if v or v == 0 or v == False}

    # def get_link(self, obj):
    #     # define your custom URL generation logic here
    #     # the `obj` parameter is the object being serialized
    #     return reverse('alert_by_id', args=[obj.identifier])

    